<?php

require_once('db_config.inc.php');

/**
    Fetches the underlying PDO object for the nacoma DB.  Connects if
    necessary, and throws a PDOException if connection fails.
*/
function pdo_db()
{
    return PDOProvider::db();
}

function pdo_print_error(PDOException $ex)
{
	bug('There was a SQL error', true, $ex->getMessage(), $ex->getTrace());
}

function pdo_error_message()
{
    try {
        $db = pdo_db();
    }
    catch(Exception $ex) {
        return $ex->getMessage();
    }
    if( $db ) {
        $errinfo = $db->errorInfo();
        return $errinfo ? $errinfo[2] : false;
    }
    else {
        return false;
    }
}

/**
 * WARNING!
 *
 * This is utterly broken absolutely everywhere, but especially so on Oracle.
 *
 * This *might* give you the last ID for the table that you inserted a new row
 * with an unspecified ID into just now, but that's only if you got lucky!
 */
function pdo_last_insert_id($tbl)
{
    $db = pdo_db();
    return $db ? $db->lastInsertId($tbl) : false;
}


/**
 * Using prepared statements is in theory faster than execute. But even more
 * important, using pdo_exec_query with non-ascii characters is sometimes
 * buggier.
 */
function pdo_prepare_statement($query, $show_message=true) {
	global $DEBUG;
	if(empty($query)) return(false);
	$db = pdo_db();
	$firebug = ($DEBUG && PHP_SAPI !== 'cli') ? true : false;
	if ($firebug) {
		require_once(dirname(__FILE__).'/../vendor/FirePHPCore/fb.php');
		FB::log($query, 'SQL Statement');
	}
	try {
		$stmt = $db->prepare($query);
	} catch (PDOException $ex) {
		if ($show_message)
			pdo_print_error($ex);
		if ($firebug)
			FB::warn($ex, 'SQL Exception');
		throw $ex;
	}
	return($stmt);
}

/**
 * Executes an SQL query, and returns the PDO result object
 *
 * If the caller handles an exception properly itself, it can set $show_message
 * to false to avoid disturbing the user.
 */
function pdo_exec_query($query, $show_message=true) {
	global $DEBUG;
	if(empty($query)) return(false);
	$db = pdo_db();
	if ($DEBUG && PHP_SAPI !== 'cli') {
		require_once(dirname(__FILE__).'/../vendor/FirePHPCore/fb.php');
		FB::log($query, 'SQL Query');
	}
	try {
		$result = $db->query($query);
	} catch (PDOException $ex) {
		if ($show_message)
			pdo_print_error($ex);
		if ($DEBUG && PHP_SAPI !== 'cli')
			FB::warn($ex, 'SQL Exception');
		throw $ex;
    }
	return($result);
}


/**
   Returns $string escaped as SQL and surrounded by single-quotes.
   If $string is-not-a string then it might be returned as-is.
 */
function pdo_quote_string($string)
{
	$db = pdo_db();
	return $db->quote($string);
}

/**
   Returns $string escaped as SQL and NOT surrounded by single-quotes.
 */
function sql_escape_string($string)
{
        $rc = pdo_quote_string($string);
        if( 0 === strpos($rc,"'") )
        { /* Workaround to accommodate older code which places its own quotes around strings. */
            $rc = substr($rc,1,strlen($rc)-2);
        }
        return $rc;
}

/**
 Equivalent to $result->fetch(PDO::FETCH_ASSOC).
*/
function pdo_fetch_array($result)
{
    return $result->fetch(PDO::FETCH_ASSOC);
}

/**
 Equivalent to $result->fetch().
*/
function pdo_fetch_row($result, $fetch_style=PDO::FETCH_BOTH)
{
    return $result->fetch($fetch_style);
}

/**
 Copies the results of the next cursor position into a associative array object
 and returns that object. Properties with SQL NULL or empty string values
 are not copied!
*/
function pdo_fetch_object($result) {
	global $DEBUG;
	if(empty($result)) {
            if($DEBUG) echo "SQL ERROR: pdo_fetch_object() called with empty resource\n";
            return(false);
	}
	$row = $result->fetch(PDO::FETCH_ASSOC);
	if ($row === false) {
            return false;
	}
	foreach($row as $k => $v) {
		if(empty($v) && $v !== 0 && $v !== '0') continue;
		$ret[$k] = $v;
	}
	return($ret);
}


/**
 Similar to pdo_fetch_object(), but returns an array of all result objects.
*/
function pdo_fetch_result($result) {
	global $DEBUG;

	if(!$result) {
		if($DEBUG) echo "SQL ERROR: pdo_fetch_result() called with empty resource\n";
		return(false);
	}
	$ret = array();
	$id = false;
	$i = 0;
	while($row = $result->fetch(PDO::FETCH_ASSOC)) {
		$i++;
		if(isset($row['id'])) {
			$id = $row['id'];
			unset($row['id']);
		}
		else $id = $i;
		if($DEBUG) {
			if(isset($ret[$id])) echo "SQL WARNING: About to overwrite row in pdo_fetch_result()\n";
		}
		foreach($row as $k => $v) {
			if(empty($v) && $v !== 0 && $v !== '0') continue;
			$ret[$id][$k] = $v;
		}
	}
	return empty($ret) ? false : $ret;
}


/**
 Counts the remaining results in $result by iterating
 over the result set, which consumes the result set.

*/
function pdo_count_rows($result) {
    $count = 0;
    while( $result->fetch() ) ++$count;
    return $count;
}
/**
 Copies all of $result to an array and then closes $result. Returns
 the array.
*/
function pdo_res_to_array($result, $fetchType = PDO::FETCH_BOTH) {
    $ary = array();
    while( ($row = $result->fetch($fetchType)) ) $ary[] = $row;
    $result->closeCursor();
    return $ary;
}

function pdo_affected_rows($ln = NULL) {
    if (!$ln)
        $ln = pdo_db();
    return $ln->rowCount();
}



