<?php
require_once('data.inc.php');
require_once(dirname(__FILE__) . '/../classes/common_gui.php');

/**
 * There are two types of locks, global and object specific ones.
 *
 * The global lock is used during exports, and means it's a really, really bad idea to even read
 * data from the database, as the database is currently being edited heavily.
 *
 * The object specific lock is acquired by a user when (s)he visits the object's editing page. It
 * merely serves as a suggestion to other users that they might create conflicts when saving the
 * object. It's not to be treated as anything beyond a suggestion, as it would otherwise, to behave
 * correctly, have to be used on all child/parent objects in $n levels, which would lock half the
 * database on every edit.
 */

/**
 * Locks the specified object from editing for $seconds
 *
 * @return true if the lock succeded, false if it was already taken
 */
function extend_lock($obj_type, $obj_id, $seconds = 60) {
	$sql = 'SELECT time - UNIX_TIMESTAMP() AS time_left FROM objlocks WHERE object_type = :otype AND object_id = :oid LIMIT 1';
	$res = pdo_prepare_statement($sql);
	$res->execute(array('otype' => $obj_type, 'oid' => $obj_id));
	$locked = $res->fetch(PDO::FETCH_NUM);
	unset($res);
	if (!empty($locked) && $locked[0] > 0) {
		// lock is still taken, refuse to steal it
		return false;
	}
	else if (!empty($locked)) {
		$sql = 'UPDATE objlocks SET username = :uname, time = UNIX_TIMESTAMP() + :seconds WHERE object_type = :otype AND object_id = :oid';
	} else {
		$sql = 'INSERT INTO objlocks (object_type, object_id, username, time) VALUES (:otype, :oid, :uname, UNIX_TIMESTAMP() + :seconds)';
	}
	$res = pdo_prepare_statement($sql);
	$res->execute(array('otype' => $obj_type, 'oid' => $obj_id, 'uname' => get_user(), 'seconds' => $seconds));
	return true;
}

function free_lock($obj_type, $obj_id) {
	$user = get_user(); // Bugs shouldn't cause users to free each other's locks
	$sql = 'DELETE FROM objlocks WHERE username = :uname AND object_type = :otype AND object_id = :oid';
	$res = pdo_prepare_statement($sql);
	$res->execute(array('otype' => $obj_type, 'oid' => $obj_id, 'uname' => $user));
}

/**
 * If this object is locked, or the whole database is locked, return the owner of the lock.
 * Otherwise, return false.
 */
function get_lock_owner($obj_type, $obj_id) {
	// $obj_id may be 'new', which we can't allow into the regular query or oracle crashes
	// As we know all non-numeric $obj_id values cannot lock, short-circut
	$user = get_user();
	if (!is_numeric($obj_id)) {
		$sql = 'SELECT username FROM objlocks WHERE object_type = \'all\' AND time > UNIX_TIMESTAMP() AND username != :uname LIMIT 1';
		$res = pdo_prepare_statement($sql);
		$res->execute(array('uname' => $user));
	} else {
		$sql = 'SELECT username FROM objlocks WHERE ((object_type = :otype AND object_id = :oid) OR object_type = \'all\') AND time > UNIX_TIMESTAMP() AND username != :uname LIMIT 1';
		$res = pdo_prepare_statement($sql);
		$res->execute(array('otype' => $obj_type, 'oid' => $obj_id, 'uname' => $user));
	}
	$val = $res->fetch(PDO::FETCH_OBJ);
	return !empty($val) ? $val->username : false;
}

/**
 * If there is a hard lock held, we need to die as soon as ve can.
 * This should thus be called pretty much at the beginning of every file.
 */
function die_if_locked_hard() {
	global $page;
	if ($owner = get_lock_owner('all', 0)) {
		$page->html_start();
		print <<<END
			<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />
			<h1>The database is currently being exported by $owner</h1>
			<p>This should merely take a few seconds, but while it's going on, no other users may edit the database.</p>
END;
		$page->html_end();
	}
}
?>
