<?php
require_once('object_class.php');

/**
 * A service runs on a host or a hostgroup. The term "service" is used very loosely. It can
 * mean an actual service that runs on the host (POP, SMTP, HTTP, etc.) or
 * some other type of metric associated with the host (response to a ping,
 * number of logged in users, free disk space, etc.).
 *
 *
 * To use it on hostgroups in the API, always specify ?parent_type=hostgroup in your calls:
 * https://monitor/api/config/service/a_hostgroup;a_service?parent_type=hostgroup
 */
 class service_class extends object_class
{
	/** type of the object */
	static $type = 'service';
	/** The type of object services must live 'under' */
	var $master_obj_type = 'host';
	var $master_obj_candidates = array('host' => 'host_name', 'hostgroup' => 'hostgroup_name');
	/** variable key identifying our 'master' object */
	var $master_obj_identifier = 'host_name';
	/** The numerical value of the host which is our 'master' */
	var $master_obj_id = false;
	/** What Nagios considers default options */
	var $nagios_defaults =
		array('stalking_options' => array('n' => ''),
		      'notification_options' => array('n' => ''),
		      'flap_detection_options' => array('o' => 1, 'c' => 1, 'w' => 1, 'u' => 1),
		      'parallelize_check' => '1',
		      'obsess' => '1',
		      'check_freshness' => '0',
		      );
	var $slave_types = array(
		'serviceextinfo',
		'serviceescalation',
		'servicedependency');

	/**
	 * Weird little hint to edit.php that during object construction, we played fast and loose with the parent type
	 * Only defined for services, because that's the only type with ambiguous parents in the name.
	 */
	public $parent_autodiscovered = false;

	/**
	 * Fetches a list of all objects sharing the same master object
	 * @returns An array of service_id => service_description entries
	 */
	function get_fellow_slaves()
	{
		if (!$this->master_obj_id)
			return false;

		$ret = array();
		$query = "SELECT id, service_description FROM service WHERE $this->master_obj_identifier = $this->master_obj_id AND id ".get_exclude_pending_string('service')." ORDER BY service_description";
		$result = pdo_exec_query($query);
		foreach ($result->fetchAll(PDO::FETCH_NUM) as $row) {
			if ($this->can_user_read('service', $row[0]))
				$ret[$row[0]] = $row[1];
		}
		unset($result);
		if ($this->master_obj_type == 'host') {
			// Magick. See explanation in object_class
			$query = "SELECT $this->db_table.id, service_description FROM $this->db_table INNER JOIN hostgroup ON $this->db_table.hostgroup_name = hostgroup.id INNER JOIN host_hostgroup ON hostgroup.id = host_hostgroup.hostgroup WHERE host_hostgroup.host = $this->master_obj_id AND $this->db_table.id ".get_exclude_pending_string($this->type)." ORDER BY service_description";
			$result = pdo_exec_query($query);
			while($row = $result->fetch(PDO::FETCH_NUM)) {
				$ret[$row[0]] = $row[1];
			}
			unset($result);
		}
		$rights_subset = get_objects_for_user('service');
		foreach ($ret as $k => $v) {
			if (!isset($rights_subset[$k]))
				unset($ret[$k]);
		}
		return $ret;
	}

	/**
	 * Validates an object by calling the parent function, as well
	 * as making sure no host gets two services with the same
	 * service_description assigned to it.
	 */
	function validate_object()
	{
		$idx = get_object_index_table($this->master_obj_type);

		$result = parent::validate_object();

		$this->validate_object_identifier();

		# templates don't need the additional checks below
		if ($this->is_template)
			return $result;

		# these are marked as 'required', so the validation error list
		# will already have them listed.
		if (empty($this->obj[$this->master_obj_identifier]) || empty($this->obj['service_description']))
			return $result;

		$query = "SELECT id FROM service WHERE $this->master_obj_identifier = " .
			pdo_quote_string($this->master_obj_id) . " AND service_description = " .
			pdo_quote_string($this->obj['service_description']);
		if ($this->id != 'new')
			$query .= " AND id != '$this->id'";

		$res = pdo_exec_query($query);
		if ($res->fetch(PDO::FETCH_NUM)) {
			$this->validation_error
				('service_description',
				 "The $this->master_obj_type '" . $idx[$this->master_obj_id] . "'" .
				 " already has a service named '" . $this->obj['service_description'] .
				 "' associated with it");
		}
		unset($res);
		return empty($this->validation_error_list) && $result;
	}

	/**
	 * Loads a service-object based on its Nagios-name
	 *
	 * @return The loaded object, or false on failures
	 */
	function get_object_by_name($name)
	{
		if (!strstr($name, ';'))
			return parent::get_object_by_name($name);

		$tmp_parts = explode(';', $name);

		$class_name = $this->master_obj_type ."_class";
		$host_obj = new $class_name();
		$host_obj = $host_obj->get_object_by_name($tmp_parts[0]);
		$master_obj_id = $host_obj['id'];
		if (!$master_obj_id && $this->master_obj_type === 'host') {
			# Nobody said anything too clear about parent type, and we couldn't find a host parent?
			# It's probably a hostgroup parent.
			$this->set_master_object_type('hostgroup');
			$obj = $this->get_object_by_name($name);
			if ($obj) {
				$this->parent_autodiscovered = true;
				return $obj;
			}
			# Give up, restore master type.
			$this->set_master_object_type('host');
			return false;
		}

		$obj_name = $tmp_parts[1];
		$query = 'SELECT id FROM ' . $this->type .
			' WHERE service_description= ' .
			pdo_quote_string($obj_name) . ' AND '.$this->master_obj_identifier.' =' . pdo_quote_string($master_obj_id);
		$result = pdo_exec_query($query);
		$row = $result->fetch(PDO::FETCH_NUM);
		unset($result);
		if (!$row) {
			# It's not a hostgroup parent, and not a host parent?
			# I bet you're mentioning the host ("monitor;PING") despite the service comming from
			# the hostgroup the host is a member of. Indirection, here I come!
			$query = 'SELECT host_hostgroup.hostgroup FROM host_hostgroup INNER JOIN host on host.id = host_hostgroup.host WHERE host.host_name = '.pdo_quote_string($tmp_parts[0]);
			$res = pdo_exec_query($query);
			$rows = $res->fetchAll(PDO::FETCH_NUM);
			unset($res);
			$hostgroups = array();
			# Maybe the host is a member of a hostgroup that is member of another hostgroup that provides the service. Woho!
			while (!empty($rows)) {
				$new_rows = array();
				foreach ($rows as $row) {
					$hostgroups[] = $row[0];
					$query = 'SELECT hostgroup FROM hostgroup_members WHERE hostgroup_members.members = '.pdo_quote_string($row[0]);
					$res = pdo_exec_query($query);
					while ($row = $res->fetch(PDO::FETCH_NUM))
						$new_rows[] = $row;
					unset($res);
				}
				$rows = $new_rows;
			}

			if (!empty($hostgroups)) {
				# What happens if there are multiple hostgroups that provide this service? I have no idea, don't do that.
				$query = 'SELECT service.id FROM service WHERE service.hostgroup_name IN ('.implode(',', $hostgroups).') AND service.service_description = '.pdo_quote_string($obj_name);
				$res = pdo_exec_query($query);
				$rows = $res->fetchAll(PDO::FETCH_ASSOC);
				unset($res);
				if ($rows) {
					$this->parent_autodiscovered = true;
					$this->id = $rows[0]['id'];
					return $this->get_object();
				}
			}
		} else {
			$this->id = $row[0];
			return $this->get_object();
		}
	}

	function denormalize($obj = false)
	{
		$obj = parent::denormalize($obj);

		if (empty($obj[$this->master_obj_identifier]))
			return $obj;

		$idx = get_object_index_table($this->master_obj_type);
		if (isset($idx[$obj[$this->master_obj_identifier]]))
			$obj[$this->master_obj_identifier] = $idx[$obj[$this->master_obj_identifier]];

		return $obj;
	}

	function save_object($user=true)
	{
		$ret = parent::save_object($user);
		if (!$ret)
			return $ret;
		$sub_objs = false;

		$dep_objs = $this->find_slave_objects('serviceextinfo');
		if (isset($dep_objs)) {
			foreach ($dep_objs as $extinfo_id) {
				$extinfo = new serviceextinfo_class($extinfo_id);
				$extinfo->banish_object();
			}
		}

		return $ret;
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone services" href="edit_special.php/copy_slaves?obj_type='.$this->type.'&amp;master_obj_id='.$this->master_obj_id.'&amp;master_obj_type='.$this->master_obj_type.'&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}

	/**
	 * Given a preexisting service id, returns the type of the parent object
	 */
	public static function get_parent_type($id) {
		$query = "SELECT 1 FROM service WHERE id = ".pdo_quote_string($id)." AND hostgroup_name IS NOT NULL AND hostgroup_name != ''";
		$res = pdo_exec_query($query);
		$row = $res->fetch(PDO::FETCH_NUM);
		if ($row)
			$parent_type = 'hostgroup';
		else
			$parent_type = 'host';
		unset($res);
		return $parent_type;
	}

	/**
	 * Export with naming and values to map nagios struct names ond format
	 *
	 * Useful for Test this x operation
	 */
	public function get_nagios_struct($prefix='') {
		$this->expand_object();
		$obj = $this->denormalize();

		return array(
			$prefix.'host_name' => $obj['host_name'],
			$prefix.'description' => $obj['service_description'],
			$prefix.'display_name' => $obj['display_name']?$obj['display_name']:$obj['service_description'],
	//		$prefix.'parents' => $obj[''],
	//		$prefix.'children' => $obj[''],
			$prefix.'check_command' => $obj['check_command'] . ($obj['check_command_args']!==""?'!'.$obj['check_command_args']:''),
			$prefix.'event_handler' => $obj['event_handler'] . ($obj['event_handler_args']!==""?'!'.$obj['event_handler_args']:''),
			$prefix.'check_interval' => $obj['check_interval'],
			$prefix.'retry_interval' => $obj['retry_interval'],
			$prefix.'max_attempts' => $obj['max_check_attempts'],
			$prefix.'parallelize' => $obj['parallelize_check'],
	//		$prefix.'contact_groups' => $obj[''],
	//		$prefix.'contacts' => $obj[''],
			$prefix.'notification_interval' => $obj['notification_interval'],
			$prefix.'first_notification_delay' => $obj['first_notification_delay'],
			$prefix.'notification_options' => $this->pack_bitmask( $obj['notification_options'], array()),
			$prefix.'stalking_options' => $this->pack_bitmask( $obj['stalking_options'], array()),
		//	$prefix.'hourly_value' => $obj[''],
			$prefix.'is_volatile' => $obj['is_volatile'],
			$prefix.'notification_period' => $obj['notification_period'],
			$prefix.'check_period' => $obj['check_period'],
			$prefix.'flap_detection_enabled' => $obj['flap_detection_enabled'],
			$prefix.'low_flap_threshold' => $obj['low_flap_threshold'],
			$prefix.'high_flap_threshold' => $obj['high_flap_threshold'],
			$prefix.'flap_detection_options' => $this->pack_bitmask( $obj['flap_detection_options'], array()),
			$prefix.'process_performance_data' => $obj['process_perf_data'],
			$prefix.'check_freshness' => $obj['check_freshness'],
			$prefix.'freshness_threshold' => $obj['freshness_threshold'],
			$prefix.'accept_passive_checks' => $obj['active_checks_enabled'],
			$prefix.'event_handler_enabled' => $obj['event_handler_enabled'],
			$prefix.'checks_enabled' => $obj['active_checks_enabled'],
		//	$prefix.'check_source' => $obj[''],
			$prefix.'retain_status_information' => $obj['retain_status_information'],
			$prefix.'retain_nonstatus_information' => $obj['retain_nonstatus_information'],
			$prefix.'notifications_enabled' => $obj['notifications_enabled'],
			$prefix.'obsess' => $obj['obsess'],
			$prefix.'notes' => $obj['notes'],
			$prefix.'notes_url' => $obj['notes_url'],
			$prefix.'action_url' => $obj['action_url'],
			$prefix.'icon_image' => $obj['icon_image'],
			$prefix.'icon_image_alt' => $obj['icon_image_alt'],
	//		$prefix.'custom_variables' => $obj[''],
	/*		$prefix.'problem_has_been_acknowledged' => $obj[''],
			$prefix.'acknowledgement_type' => $obj[''],
			$prefix.'host_problem_at_last_check' => $obj[''],
			$prefix.'check_type' => $obj[''],
			$prefix.'current_state' => $obj[''],
			$prefix.'last_state' => $obj[''],
			$prefix.'last_hard_state' => $obj[''],
			$prefix.'plugin_output' => $obj[''],
			$prefix.'long_plugin_output' => $obj[''],
			$prefix.'perf_data' => $obj[''],
			$prefix.'state_type' => $obj[''],
			$prefix.'next_check' => $obj[''],
			$prefix.'should_be_scheduled' => $obj[''],
			$prefix.'last_check' => $obj[''],
			$prefix.'current_attempt' => $obj[''],
			$prefix.'current_event_id' => $obj[''],
			$prefix.'last_event_id' => $obj[''],
			$prefix.'current_problem_id' => $obj[''],
			$prefix.'last_problem_id' => $obj[''],
			$prefix.'last_notification' => $obj[''],
			$prefix.'next_notification' => $obj[''],
			$prefix.'no_more_notifications' => $obj[''],
			$prefix.'check_flapping_recovery_notification' => $obj[''],
			$prefix.'last_state_change' => $obj[''],
			$prefix.'last_hard_state_change' => $obj[''],
			$prefix.'last_time_ok' => $obj[''],
			$prefix.'last_time_warning' => $obj[''],
			$prefix.'last_time_unknown' => $obj[''],
			$prefix.'last_time_critical' => $obj[''],
			$prefix.'has_been_checked' => $obj[''],
			$prefix.'is_being_freshened' => $obj[''],
			$prefix.'notified_on' => $obj[''],
			$prefix.'current_notification_number' => $obj[''],
			$prefix.'current_notification_id' => $obj[''],
			$prefix.'latency' => $obj[''],
			$prefix.'execution_time' => $obj[''],
			$prefix.'is_executing' => $obj[''],
			$prefix.'check_options' => $obj[''],
			$prefix.'scheduled_downtime_depth' => $obj[''],
			$prefix.'pending_flex_downtime' => $obj[''],
			$prefix.'is_flapping' => $obj[''],
			$prefix.'flapping_comment_id' => $obj[''],
			$prefix.'percent_state_change' => $obj[''],
			$prefix.'modified_attributes' => $obj[''],

			$prefix.'host_ptr' => $obj[''],
			$prefix.'event_handler_ptr' => $obj[''],
			$prefix.'event_handler_args' => $obj[''],
			$prefix.'check_command_ptr' => $obj[''],
			$prefix.'check_command_args' => $obj[''],
			$prefix.'check_period_ptr' => $obj[''],
			$prefix.'notification_period_ptr' => $obj[''],
			$prefix.'servicegroups_ptr' => $obj[''],*/
				$prefix.'custom_variables' => $this->extract_custom_vars($obj)
				);
	}
}
