<?php

require_once('devtools/tapunit.php');
require_once('classes/object_class.php');
if (empty($main_tap))
	$main_tap = new phptap();

class AccessTest extends TapUnit
{
	# All scripts have a bad habit of running exit() when they fail.
	# Therefore, when running pages, run them in a subprocess.
	function run_page($filename, $group, $args) {
		$user = get_user();
		$res = shell_exec('php test/phptap/websiterunner.php '.$filename.' '.$user.' '.$group.' '.$args);
		#print $res;
		return $res;
	}

	static function tearDownClass() {
		set_user('contact_limited', array('services_only'));
		revert_users_changes();
		pdo_exec_query('DELETE FROM changelog');
	}

	function testUserRightExtraction()
	{
		set_user('host_7', array('op5_admins'));
		$rights = get_rights();
		$this->ok(is_array($rights), "get_rights returned array");
		$this->ok(isset($rights['system_information']), "user can see system info");
		$this->ok(isset($rights['configuration_information']), "user can see config info");
		$this->ok(isset($rights['host_view_all']), "user can see all hosts");
		set_user('contact1', array('contact1'));
		$rights = get_rights();
		$this->ok(isset($rights['system_information']), "user can see system info");
		$this->ok(!isset($rights['host_view_all']), "user can't see all hosts");
		set_user('nonexisting', array('nonexisting'));
		$rights = get_rights();
		$this->ok(count($rights) === 0, "No rights array for non-user");
	}

	function testUserAccess()
	{
		set_user('nonexisting', array('nonexisting'));
		$res = $this->run_page('index.php', 'nonexisting', '');
		$this->ok(strpos($res, "Access denied") !== false, "Nonexisting user can't access interface");
		$res = $this->run_page('edit.php', 'nonexisting', "obj_type host obj_id 2");
		$this->ok(strpos($res, "Access denied") !== false, "Nonexisting user can't access interface");

		set_user('host_7', array('op5_admins'));
		$nhosts = 56;
		$res = $this->run_page('index.php', 'op5_admins', '');
		$this->ok_eq(substr_count($res, ">host_"), $nhosts, "All $nhosts hosts visible to admin in cfg");
		$res = $this->run_page('edit.php', 'op5_admins', 'obj_type host host host_1');
		$this->ok_eq(substr_count($res, ">host_"), $nhosts*2+5, "All $nhosts hosts visible to admin in edit");

		set_user('contact_limited', array('contact_limited'));
		$nhosts = 4;
		$res = $this->run_page('index.php', 'contact_limited', '');
		$this->ok_eq(substr_count($res, '>host_'), $nhosts, "Only $nhosts hosts should be visible to limited in cfg, expected $nhosts matches, got ".substr_count($res, '>host_'));
		$res = $this->run_page('edit.php', 'contact_limited', 'obj_type host host host_1');
		$this->ok_eq(substr_count($res, ">host_"), $nhosts*2+5, "Only $nhosts hosts visible to limited in edit");
		$res = $this->run_page('edit.php', 'contact_limited', 'obj_type host host host_3');
		$this->ok(strpos($res, "Need help") === false, "Limited shouldn't be allowed to access other's hosts");
		$res = $this->run_page('edit.php', 'contact_limited', 'obj_type service host host_4 service PING');
		$this->ok(strpos($res, "Need help") === false, "Limited isn't allowed to see other's services");
		$res = $this->run_page('edit.php', 'contact_limited', 'obj_type service host host_1 service Domain');
		$this->ok(strpos($res, "Access denied") === false, "Limited is allowed to see it's services, got access denied");
		$res = $this->run_page('edit.php', 'contact_limited', 'obj_type service obj_id 191');
		$this->ok(strpos($res, "Access denied") !== false, "Limited isn't allowed to see other's services by id");

		set_user('contact2', array('contact2'));
		$nhosts = 56;
		$res = $this->run_page('index.php', 'op5_admins', '');
		$this->ok_eq(substr_count($res, ">host_"), $nhosts, "All $nhosts hosts visible to admin without explicit rights in cfg");
		$res = $this->run_page('edit.php', 'op5_admins', 'obj_type host host host_1');
		$this->ok_eq(substr_count($res, ">host_"), $nhosts*2+5, "All $nhosts hosts visible to admin without explicit rights in edit");
	}

	// yes, this test really should be split up.
	function testValidateUserRights()
	{
		set_user('contact_limited', array('contact_limited'));
		$obj = new host_class('host_1');
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to store his host');

		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to store his updated host');

		$obj = new host_class('host_2');
		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), 'User isn\'t allowed to store others\' host');

		set_user('contact_limited', array('contact_limited', 'contact_limited_add_delete'));
		$orig_obj = new host_class('host_1');
		$obj = $orig_obj->clone_object();
		$obj->obj['host_name'] = 'foo';
		$obj->obj['max_check_attempts'] = '1';
		$obj->obj['notification_interval'] = '1';
		$obj->obj['notification_period'] = '1';
		$obj->obj['notification_options'] = 'd';
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to add new hosts');

		$obj = new service_class('host_1;Domain');
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to store his services');
		$obj = $obj->clone_object();
		$obj->obj['service_description'] = 'foo';
		$obj->obj['host_name'] = 3;
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to add new services');

		set_user('contact_limited', array('contact_limited'));
		$obj = new service_class('host_2;Domain');
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), 'User isn\'t allowed to store others\' services');

		# The user is only allowed to modify relations between his own objects. This
		# means all other changes are reverted. In some situations, this might be
		# unexpected.
		$obj = new host_class('host_1');
		$oldp = $obj->obj['parents'];
		sort($oldp);
		$this->ok(!empty($oldp), 'Host has the parent property set');
		$obj->obj['parents'] = array();
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to store his host with modified relations');
		$newp = $obj->obj['parents'];
		sort($newp);
		$this->ok_eq($newp, $oldp, 'User can\'t manipulate relations to others\' objects');

		$obj->obj['parents'] = array(15);
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to store his host with modified relations');
		$newp = $obj->obj['parents'];
		sort($newp);
		$this->ok_eq($newp, $oldp, 'User can\'t add relations to others\' objects');

		$otherobj = new host_class('host_10');
		$obj->obj['parents'] = array($otherobj->id);
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User is allowed to store his host with modified relations');
		$newp = $obj->obj['parents'];
		sort($newp);
		$correctp = $oldp;
		$correctp[] = $otherobj->id;
		sort($correctp);
		$this->ok_eq($newp, $correctp, 'User can add relations between his objects');

		set_user('host_7', array('op5_admins'));

		$obj = new timeperiod_class(1);
		$obj->obj['alias'] = 'foobar';
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), "User is allowed to modify 'allow' objects");

		$obj = $obj->clone_object();
		$obj->obj['timeperiod_name'] = 'foo';
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), "User is allowed to create 'allow' objects");

		set_user('host_7', array('op5_admins'));

		$obj = new timeperiod_class(1);
		$obj->obj['alias'] = 'foobar';
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), "User is allowed to modify 'allow' objects");

		set_user('contact_limited', array('contact_limited'));

		$obj = $obj->clone_object();
		$obj->obj['timeperiod_name'] = 'foo';
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), "User isn't allowed to create 'allow' objects");

		$obj = new timeperiod_class(1);
		$obj->obj['alias'] = 'foobar';
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), "User isn't allowed to modify 'allow' objects");

		$obj = new timeperiod_class(1);
		$obj->obj['alias'] = 'foobar';
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), "User isn't allowed to modify 'allow' objects");

		$obj = $obj->clone_object();
		$obj->obj['timeperiod_name'] = 'foo';
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), "User isn't allowed to create 'allow' objects");

		set_user('contact2', array('contact2'));

		// TODO: Check that the rights match this test.
		$obj = new host_class('host_1');
		$obj->obj['parents'] = array();
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'Admin user is allowed to store others\' host with modified relations');
		$newp = $obj->obj['parents'];
		$this->ok_eq($newp, array(), 'Admin user can manipulate relations to others\' objects');
		revert_users_changes();
		set_user('contact_limited', array('contact_limited'));
		revert_users_changes();
	}

	function testAdditionalValidateUserRights()
	{
		// This verifies that bug #6667 is solved.
		set_user('contact_limited', array('contact_edit_view_all'));
		$obj = new host_class('host_1');
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), "User should be able to save his own host but wasn't");

		$obj = new host_class('host_2');
		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), 'User shouldn\'t be allowed to store others\' host but was');

		/**
		 * If one of the next two tests fail but not the previous two, add_delete gives more permissions than it should
		 * Tip! Check functions get_objects_for_user() and can_user_write().
		 */
		set_user('contact_limited', array('contact_edit_view_all_ad'));
		$obj = new host_class('host_1');
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), "User should be able to save his own host but wasn't");

		$obj = new host_class('host_2');
		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->ok(!empty($obj->validation_error_list), 'User shouldn\'t be allowed to store others\' host but was');
		// End tests for bug #6667

		// Strange way of setting permissions on a user but hey, who are we to judge
		set_user('contact_limited', array('view_contact_edit_all'));
		$obj = new host_class('host_1');
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), "User should be able to save his own host but wasn't");

		// Technically this user should be able to edit another's host, however this host will not show up anywhere.
		$obj = new host_class('host_2');
		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->ok(empty($obj->validation_error_list), 'User should be allowed to store others\' host but wasn\'t');
	}

	function testIsUserContact()
	{
		require_once(dirname(__FILE__).'/../../ajax_helper.php');
		set_user('host_7', array('op5_admins'));
		$this->ok(is_user_contact('host', array(), array(), false), 'Superusers have access to any host object');
		$this->ok(is_user_contact('service', array(), array(), false), 'Superusers have access to any service object');
		set_user('contact1', array('contact1'));
		$this->ok(!is_user_contact('host', array(), array(), false), "Limited users don't have access to any host object");
		$this->ok(!is_user_contact('service', array(), array(), false), "Limited users don't have access to any service object");
		$user_obj = new contact_class('contact1');
		$uid = $user_obj->id;
		$this->ok(is_user_contact('host', array($uid), array(), false), "Limited users that are contacts can access hosts");
		$this->ok(is_user_contact('service', array($uid), array(), false), "Limited users that are contacts can access services");
		$cgroup = new contactgroup_class('support');
		$cgid = $cgroup->id;
		$this->ok(is_user_contact('host', array(), array($cgid), false), "Limited users that are contacts can access hosts");
		$this->ok(is_user_contact('service', array(), array($cgid), false), "Limited users that are contacts can access services");
	}

	function testUserCanOnlyEditService()
	{
		set_user('host_7', array('op5_admins'));
		$ug = new usergroup_class('services_only');
		$ug->obj['access_rights'] = array('service_add_delete', 'service_view_contact', 'service_edit_contact', 'host_view_contact', 'service_template_view_all', 'configuration_information', 'command_view_all');
		$this->ok($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));
		set_user('contact_limited', array('services_only'));
		$host = new host_class('host_10');
		# rely on PHP's ability to cast "new" to 0:
		$this->ok($host->id > 0, "Load contact's host should be fine");
		$service = new service_class();
		$service->obj['host_name'] = $host->id;
		$service->obj['service_description'] = 'test_service';
		$service->normalize(array('contacts' => array('contact_limited')));
		$service->obj['template'] = 1;
		$service->obj['check_command'] = 1;
		$this->ok($service->save_object(), 'Creating new service should work, but got: '.var_export($service->validation_error_list, true));
		$this->ok(!isset($service->obj['contacts']), 'The service should have the contacts removed when we have no contact rights');
		$this->ok_id($service->obj['host_name'], $host->id, 'The service should still have the host_name when we have host view rights');
		$service->obj['notes'] = 'This is a note';
		$this->ok($service->save_object(), 'Editing service should work, but got: '.var_export($service->validation_error_list, true));
		$res = $this->run_page('edit.php', 'services_only', "obj_type service obj_id ".$service->id);
		$this->ok(strpos($res, 'Access denied') === false, "The web ui should not restrict access to see service on host we're authorized to see, said: ".$res);
		$this->ok($service->banish_object(), 'Deleting service should work, but got: '.var_export($service->validation_error_list, true));
		$this->ok(!$host->save_object(), 'Saving host we can only view should not work');

		$host = new host_class('host_20');
		$this->ok(!$host->save_object(), "Saving host we can't access shouldn't work");
		$service = new service_class();
		$service->obj['host_name'] = $host->id;
		$service->obj['service_description'] = 'test_service';
		$service->obj['template'] = 1;
		$service->obj['check_command'] = 1;
		$this->ok($service->save_object(), "Saving services on hosts we can't see shouldn't work");
	}

	function testUserRedirectViaHostgroup()
	{
		$svc = new service_class('host_4;metaservice');
		$this->ok(!empty($svc->obj), "The service should be found");
		$this->ok_eq($svc->obj['service_description'], 'metaservice', "The requested description should be set");
		$hg = new hostgroup_class('hostgroup_meta');
		$this->ok_eq($svc->obj['hostgroup_name'], $hg->id, "The host should be replaced with the true hostgroup");
		set_user('host_7', array('op5_admins'));
		$res = $this->run_page('edit.php', 'op5_admins', 'obj_type service host host_4 service metaservice');
		$this->ok(strpos($res, 'hostgroup_meta;metaservice') !== false, 'Service should be located on hostgroup, even indirectly');
		$this->ok(strpos($res, 'You were trying to reach a host service, but it is actually a hostgroup service') !== false, 'An appropriate warning should be displayed post redirect');
	}

	function test_user_can_not_delete_without_delete()
	{
		set_user('contact_limited', array('contact_limited'));
		$host = new host_class('host_1');
		$this->ok(!!$host->id, 'Loading host_1 with view_all should work');
		$this->ok($host->save_object(), 'Saving host_1 with edit_contact should work');
		$this->ok(!$host->banish_object(), 'Banishing host_1 with edit_contact should not work');

		$host = new host_class('host_12');
		$this->ok(!!$host->id, 'Loading host_12 with view_all should work');
		$this->ok(!$host->save_object(), 'Saving host_12 with edit_contact should not work');
		$this->ok(!$host->banish_object(), 'Banishing host_12 with edit_contact should not work');

		set_user('host_7', array('op5_admins'));
		$ug = new usergroup_class('delete_limited');
		$ug->obj['access_rights'] = array('host_add_delete', 'host_view_all', 'host_edit_contact', 'service_add_delete', 'service_edit_contact', 'servicedependency_add_delete', 'servicedependency_edit_all', 'hostdependency_add_delete', 'hostdependency_edit_all');
		$this->ok($ug->save_object(), 'New usergroup should save');
		set_user('contact_limited', array('delete_limited'));
		$host = new host_class('host_1');
		$this->ok(!!$host->id, 'Loading host_1 with view_all should work');
		$this->ok($host->save_object(), 'Saving host_1 with edit_contact should work');
		$obj = $host->obj;
		$this->ok($host->banish_object(), 'Banishing host_1 with add_delete+edit_contact should work, got: ' . $host->delete_message);

		$host = new host_class('host_12');
		$this->ok(!!$host->id, 'Loading host_12 with view_all should work');
		$this->ok(!$host->save_object(), 'Saving host_12 with edit_contact should not work');
		$this->ok(!$host->banish_object(), 'Banishing host_12 with add_delete+edit_contact should not work');

		revert_users_changes();

		set_user('host_7', array('op5_admins'));
		$host = new host_class('host_1');
	}
}

runUnit('AccessTest', $main_tap);
?>
