<?php

require_once('devtools/tapunit.php');
require_once('classes/object_class.php');

/**
 * Test higher level functionality in object_class
 * Lower level tests are in oatest.php
 */
class ObjectTest extends TapUnit
{
	function testTemplateDeletion()
	{
		$tmpl = new host_template_class();
		$tmpl->get_default_object();
		$tmpl->obj['name'] = 'tmp_tmpl';
		$tmpl->obj['check_command'] = $tmpl->obj['notification_period'] = 1;
		$tmpl->save_object();
		$this->ok_empty($tmpl->validation_error_list, 'Template created');
		$tmpl_id = $tmpl->id;

		$host = new host_class();
		$host->get_default_object();
		$host->obj['host_name'] = $host->obj['alias'] = $host->obj['address'] = 'tmphost';
		$host->obj['template'] = $tmpl->id;
		$host->save_object();
		$this->ok_empty($host->validation_error_list, 'Host created');
		$host_id = $host->id;

		$this->ok(!$tmpl->banish_object(), "Template isn't deleted because it has an active object user");
		$this->ok($host->banish_object(), "Host is deleted");
		$this->ok($tmpl->banish_object(), "Template is deleted when host is gone");
		
		$tmpl = new host_template_class($tmpl_id);
		$this->ok_empty($tmpl->obj, 'Template deleted');

		$host = new host_class($host_id);
		unset($host->obj['contacts']);
		$this->ok_empty($host->obj, 'Host deleted when it\'s template is');
	}

	function testCopyService()
	{
		$srv = new service_class();
		$srv->get_default_object();
		$srv->obj['host_name'] = 1;
		$srv->obj['service_description'] = 'testservice';
		$srv->obj['check_command'] = 1;
		$srv->save_object();
		$this->ok_empty($srv->validation_error_list, 'Service saved ok');

		$esc = new serviceescalation_class();
		$esc->get_default_object();
		$esc->obj['service'] = $srv->id;
		$esc->save_object();
		$this->ok_empty($esc->validation_error_list, 'Escalation saved ok');

		$new_srv = $srv->clone_object();
		$new_srv->obj['service_description'] = 'cloned testservice';
		$this->ok_eq($new_srv->clone_id, $srv->id, 'New service has old service\'s id as clone_id');
		$tmp = $srv->find_slave_objects();
		$this->ok_eq($tmp['serviceescalation'][0], $esc->id, 'Old service has the new escalation');
		$this->ok_empty($new_srv->find_slave_objects(), 'No dependant objects yet');
		$new_srv->save_object();
		$this->ok_empty($new_srv->validation_error_list, 'New service saved ok');
		$tmp = $srv->find_slave_objects();
		$this->ok_eq($tmp['serviceescalation'][0], $esc->id, 'Old service still has the new escalation');
		$tmp2 = $new_srv->find_slave_objects();
		$this->ok(!empty($tmp2['serviceescalation']), 'New service has its own service escalation');
		$this->ok($tmp2['serviceescalation'][0] != $tmp['serviceescalation'][0], 'Different escalations are assigned to the two services');
	}

	function testServiceNoCommand() {
		# test both that this works and doesn't
		# because I don't want to pass tests because, say, the host name is invalid
		foreach (array(1, false) as $cmd) {
			$srv = new service_class();
			$srv->get_default_object();
			$srv->obj['service_description'] = 'a service';
			$srv->obj['host_name'] = 1;
			$srv->obj['check_command'] = $cmd;
			$this->ok_eq(!!$cmd, $srv->save_object(), sprintf("Save new object with %s command should %swork", ($cmd ? "valid" : "invalid"), ($cmd ? "" : "not ")));
			if ($cmd) {
				$this->ok_empty($srv->validation_error_list, "Validation error list should be empty after saving worked");
				$this->ok($srv->banish_object(), "Deleting the newly created object should work");
			}
		}
	}

	function testExtinfo() {
		$hextinfo = new hostextinfo_class(41);
		$this->ok(!empty($hextinfo->obj), 'Hostextinfo exists');
		$host = new host_class($hextinfo->obj['host_name']);
		$host->save_object();
		$this->ok_empty($host->validation_error_list, 'Host saved ok');
		$new_hostextinfo = new hostextinfo_class($hextinfo->id);
		$this->ok_empty($new_hostextinfo->obj, 'Hostextinfo no longer exists');
		$this->ok_eq($hextinfo->obj['notes_url'], $host->obj['notes_url'], "Notes URL moved to master object");
		$this->ok_eq($hextinfo->obj['icon_image'], $host->obj['icon_image'], "Icon image moved to master object");
		$this->ok_eq($hextinfo->obj['icon_image_alt'], $host->obj['icon_image_alt'], "Icon image alt moved to master object");
		$this->ok_eq($hextinfo->obj['statusmap_image'], $host->obj['statusmap_image'], "Statusmap image moved to master object");
	}

	function group_of_groups_helper($type, $groupmember, $member) {
		$class = $type.'_class';
		$obj = new $class();
		$obj->get_default_object();
		$obj->obj[$type.'_name'] = 'test '.$type;
		$obj->obj['alias'] = 'test '.$type;
		$this->ok($obj->save_object(), "Tried to save a {$type} with no members, but failed");
		$obj->obj['members'] = array($member);
		$this->ok($obj->save_object(), "Saving is successful when members are added to the {$type}");
		$reloaded_obj = new $class($obj->id);
		$this->ok_eq($reloaded_obj->obj['members'], $obj->obj['members'], 'Members are saved correctly');
		$this->ok(!isset($reloaded_obj->obj[$type.'_members']), "No {$type}_members added");
		$obj->obj['members'] = array();
		$obj->obj[$type.'_members'] = array($groupmember);
		$this->ok($obj->save_object(), "Saving is successful when {$type}_members");
		$reloaded_obj = new $class($obj->id);
		$this->ok_eq($reloaded_obj->obj[$type.'_members'], $obj->obj[$type.'_members'], "{$type}_members are saved correctly");
		$this->ok(!isset($reloaded_obj->obj['members']), "No members added");

		$clone = $obj->clone_object();
		$this->ok_eq($clone->obj[$type.'_members'], $obj->obj[$type.'_members'], "Cloned object retains {$type}_members attribute");
		$id = $obj->id;
		$obj->banish_object();
		$new_obj = new $class($id);
		$this->ok_empty($new_obj->obj, 'Object was really deleted');
	}

	function testGroupOfGroups() {
		$this->group_of_groups_helper('hostgroup', 1, 1);
		$this->group_of_groups_helper('servicegroup', 1, 1);
		$this->group_of_groups_helper('contactgroup', 1, 1);
	}

	function template_helper($type) {
		$props = $type == 'contact' ? array('address1', 'address2') : array('notes', 'action_url');

		$obj_class = $type.'_class';
		$tmpl_class = $type.'_template_class';
		$gptmpl = new $tmpl_class();
		$gptmpl->obj['name'] = 'Grandparent test template';
		$gptmpl->obj[$props[0]] = "This is a {$props[0]} from grandparent";
		$this->ok($gptmpl->save_object(), "Saving grandparent $type template should work, got " . var_export($gptmpl->validation_error_list, true));
		$ptmpl = new $tmpl_class();
		$ptmpl->obj['name'] = 'Parent test template';
		$ptmpl->obj[$props[1]] = 'http://schwarzenegger.com';
		$ptmpl->obj['template'] = $gptmpl->id;
		$this->ok($ptmpl->save_object(), "Saving parent $type template should work, got " . var_export($ptmpl->validation_error_list, true));
		$obj = new $obj_class();
		$obj->obj['template'] = $ptmpl->id;
		if ($type == 'service') {
			$obj->obj['host_name'] = 'host_9';
			$obj->obj['service_description'] = 'test service';
			$obj->obj['check_interval'] = $obj->obj['retry_interval'] = $obj->obj['check_period'] = $obj->obj['check_command'] = 1;
		} else {
			$obj->obj['alias'] = $obj->obj[$type.'_name'] = 'test '.$type;
		}
		if ($type !== 'contact')
			$obj->obj['max_check_attempts'] = $obj->obj['notification_interval'] = $obj->obj['notification_period'] = 1;
		$this->ok($obj->save_object(), "Saving the $type object should work, got " . var_export($obj->validation_error_list, true));
		$obj = new $obj_class($obj->id);
		$obj->expand_object(true);
		$ptmpl->expand_object(true);
		$gptmpl->expand_object(true);
		$this->ok_eq($obj->obj[$props[1]], $ptmpl->obj[$props[1]], $type . ' action url should be set from direct template');
		$this->ok_eq($obj->obj[$props[0]], $gptmpl->obj[$props[0]], $type . ' notes should be set from indirect template');
		$this->ok_eq($ptmpl->obj[$props[0]], $gptmpl->obj[$props[0]], $type . ' notes should be set to template from parent template');

		$this->ok(!$gptmpl->delete_object(), 'Deleting grandparent should not work, but it did: ' .$gptmpl->delete_message);
		$this->ok(!$ptmpl->delete_object(), 'Deleting parent should not work, but it did: ' .$ptmpl->delete_message);
		$this->ok($obj->delete_object(), "Deleting object should now work, but it didn't: " .$obj->delete_message);
		$this->ok($ptmpl->delete_object(), "Deleting parent should now work, but it didn't: " .$ptmpl->delete_message);
		$this->ok($gptmpl->delete_object(), "Deleting grandparent should now work, but it didn't: " .$gptmpl->delete_message);
	}

	function testTemplateTemplate() {
		$this->template_helper('host');
		$this->template_helper('service');
		$this->template_helper('contact');
	}

	function testResaveServiceWithSemicolons() {
		$svc = new service_class('host_9;HTTP Server');
		$svc->save_object();
		$this->ok_empty($svc->validation_error_list, 'Resave service with escaped ; ok');
		$svc->obj['check_command'] = 'check_http!arg;with;unescaped;semicolons';
		$this->ok(!$svc->save_object(), 'Save service with unescaped ; not ok');
		$svc->obj['check_command'] = 'check_http!arg\;with;semicolons';
		$this->ok(!$svc->save_object(), 'Save service with some unescaped ; not ok');
	}

	function testSaveAndPermissions() {
		$host = new host_class('host_9');
		$svc = new service_class();
		$svc->obj['host_name'] = $host->id;
		$svc->obj['service_description'] = 'testservice';
		$svc->obj['template'] = 1;
		$svc->obj['check_command'] = 1;
		$svc->save_object();
		$svc->prepare_draw();
		$this->ok_empty($svc->validation_error_list, 'New service saved');
		$fellow_slaves = $svc->get_fellow_slaves();
		$this->ok(count($fellow_slaves) > 1, 'The list of fellow slaves contains objects other than this');
		$old_svc = new service_class('host_9;HTTP Server');
		$this->ok(in_array($old_svc->id, array_keys($fellow_slaves)), 'The HTTP Server service on this host is a fellow slave');
		$svc->banish_object();
	}

	/**
	 * bug #6488
	 */
	function testPropagateAppend() {
		$host = new host_class('host_9');
		$host->propagate_variables(array('hostgroups' => $host->obj['hostgroups']), array('host_11', 'host_23'), 'append');
		$propagated = new host_class('host_23');
		$this->ok_eq(count($propagated->obj['hostgroups']), 2, 'host_23 should have its original + host_9\'s hostgroup = 2');
		$this->ok(!in_array('hostgroup_4', $propagated->obj['hostgroups']), 'host_23 should not get host_11s old hostgroups');
	}

	function testPropagateSubtract() {
		$host = new host_class('host_9');
		$host->propagate_variables(array('hostgroups' => $host->obj['hostgroups']), array('host_11', 'host_23'), 'subtract');
		$propagated = new host_class('host_23');
		$this->ok_eq(count($propagated->obj['hostgroups']), 1, 'host_23 should no longer have host_9\'s hostgroup => 1');
		$this->ok_eq(count($propagated->obj['hostgroups']), 1, 'host_11 should no longer have host_9\'s hostgroup => 1');
	}

	/**
	 * bug #6661
	 */
	function testTemplateCustomVar() {
		$tmpl = new host_template_class('default-host-template');
		$this->ok_eq($tmpl->custom_vars['_random_var'], 'predictable_value', 'The template should have the expected custom variable');
		$hst1 = new host_class('host_1');
		$this->ok_eq($hst1->custom_vars['_random_var'], 'not_a_random_value', 'The object that overrides template should have its custom variable');
		$hst2 = new host_class('host_2');
		$this->ok_empty($hst2->custom_vars, "The object that doesn't override template shouldn't have a custom variable");
		$hst2->expand_object();
		$this->ok_eq($hst2->custom_vars['_random_var'], 'predictable_value', "The object that doesn't override template should load templates custom variable when asked");
		$this->ok($hst1->save_object(), 'Saving unmodified object should work');
		$this->ok($hst2->save_object(), 'Saving unmodified object should work');
		$tmpl->custom_vars['_random_var'] = 'new value';
		$this->ok($tmpl->save_object(), 'Saving template with new custom var should work');
		$tmpl = new host_template_class('default-host-template');
		$this->ok_eq($tmpl->custom_vars['_random_var'], 'new value', 'The template should have the expected custom variable');
		$hst1 = new host_class('host_1');
		$this->ok_eq($hst1->custom_vars['_random_var'], 'not_a_random_value', 'The object that overrides template should still have its custom variable');
		$hst2 = new host_class('host_2');
		$this->ok_empty($hst2->custom_vars, 'The object that doesn\'t override template should still not have a custom variable');
		$hst2->expand_object();
		$this->ok_eq($hst2->custom_vars['_random_var'], 'new value', 'The object that doesn\'t override template should still load templates custom variable when told');
	}

	function testUnicode() {
		$strings = array(
			'今年から貝が胃に棲み始めました。' => true, #kanji - with scary full stop
			'আমার হভারক্রাফ্ট ইল-এ ভর্তি হয়ে' => true, #bengal - scary all over, with "vowel stop" and nastiness
			'  ' => true, #klingon - private plane
			"Looks alright (but isn't)" => false, #parenthesis
			"Now we backspace\x08" => false, #backspace character
		);

		$host = new host_class('new host');
		$host->get_default_object();
		foreach ($strings as $string => $legal) {
			$host->obj['host_name'] = $string;
			$this->ok($legal == $host->save_object(), "Setting the identifier of a host to '$string' should" . ($legal ? " be allowed, but wasn't" : "n't be allowed, but was") . ": " . var_export($host->validation_error_list, true));
		}

		$service = new service_class();
		$service->obj['template'] = 1;
		$service->obj['check_command'] = 1;
		$service->obj['host_name'] = $host->id;
		foreach ($strings as $string => $legal) {
			$service->obj['service_description'] = $string;
			$this->ok($legal == $service->save_object(), "Setting the identifier of a service to '$string' should" . ($legal ? " be allowed, but wasn't" : "n't be allowed, but was") . ": " . var_export($service->validation_error_list, true));
		}
	}

	function testContactWithUser() {
		$contact = new contact_class();
		$contact->obj['enable_access'] = 1;
		$contact->obj['alias'] = $contact->obj['contact_name'] = 'test_contact_with_user';
		$contact->obj['password'] = 'test_password';
		$contact->obj['groups'] = array('contact_limited');
		$this->ok($contact->save_object(), "Saving contact with test user should work, got " . var_export($contact->validation_error_list, true));
		$user = new user_class('test_contact_with_user');
		$this->ok_eq($user->obj['groups'], $contact->obj['groups'], "Group list for loaded user should be equal to group list for contact-with-user");
	}

	function testDeleteNumericUser() {
		$user = new user_class();
		$user->obj['username'] = '0123456789';
		$user->obj['realname'] = 'numeric';
		$user->obj['password'] = 'test_password';
		$user->obj['groups'] = array('contact_limited');
		$this->ok($user->save_object(), "Saving user with numeric name should work, got " . var_export($user->validation_error_list, true));
		user_class::drop_cache();
		$user = new user_class('0123456789');
		$this->ok_eq($user->obj['username'], '0123456789', 'The username should still have the leading zero');
		$this->ok_eq($user->obj['groups'], array('contact_limited'), "Group list for loaded user should be equal to group list for contact-with-user");
		$this->ok($user->banish_object(), "Banishing the newly created numeric user should work");
		$user = new user_class('0123456789');
		$this->ok_empty($user->obj, "Banished numeric user should be gone");
	}

	function testManagementPack() {
		$mp = new management_pack_class();
		$mp->obj['hostgroup'] = 'hostgroup_1';
		$mp->obj['management_pack_name'] = 'the management pack';
		$mp->obj['description'] = 'a management pack';
		$mp->obj['icon'] = 'edit.png';
		$this->ok($mp->save_object(), 'Saving a new management pack should be OK');
		$this->ok_empty($mp->validation_error_list, 'There should be no validation errors when saving new pack');
		$old_mp = new management_pack_class('the management pack');
		$this->ok(!empty($old_mp->obj), 'Loading a saved management pack should work');
		$this->ok(is_numeric($old_mp->id), 'There should be a numeric id for a saved management pack');
		$this->ok($old_mp->id > 0, 'The saved management pack should have an ID greater than 0');
		$this->ok_eq($old_mp->obj['management_pack_name'], 'the management pack', 'The saved name should be loaded');
		$this->ok_eq($old_mp->obj['hostgroup'], 'hostgroup_1', "The original, non-normalized hostgroup name should be loaded again");
		$old_mp->obj['management_pack_name'] = 'new name';
		$old_mp->custom_vars['_DBNAME'] = 'This names a database';
		$this->ok($old_mp->save_object(), 'Renaming the changed management pack should work');
		$new_mp = new management_pack_Class('new name');
		$this->ok(!empty($new_mp->obj), 'Loading a edited management pack should work');
		$this->ok(is_numeric($new_mp->id), 'There should be a numeric id for a edited management pack');
		$this->ok($new_mp->id > 0, 'The edited management pack should have an ID greater than 0');
		$this->ok_eq($new_mp->obj['management_pack_name'], 'new name', 'The edited name should be loaded');
		$this->ok_eq($new_mp->custom_vars['_DBNAME'], 'This names a database', 'The saved custom var should be loaded');
		$this->ok($new_mp->banish_object(), 'Deleting the management pack should work');
		$this->ok(is_array($new_mp->delete_message), "When deleting the management pack, there shouldn't be an error message, was " . $new_mp->delete_message);
	}

	function testRenameHostgroupWithMgmtPack() {
		$hg = new hostgroup_class();
		$hg->obj['hostgroup_name'] = 'hostgroup before rename';
		$this->ok($hg->save_object(), 'Saving new hostgroup should work');
		$this->ok_empty($hg->validation_error_list, 'The validation error list for new hostgroup should be empty');

		$mgmt = new management_pack_class();
		$mgmt->obj['management_pack_name'] = 'a mgmt pack';
		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$mgmt->obj['description'] = 'a management pack';
		$mgmt->obj['icon'] = 'edit.png';
		$this->ok($mgmt->save_object(), 'Saving new mgmt pack should work');
		$this->ok_empty($mgmt->validation_error_list, 'The validation error list for new mgmt pack should be empty');

		$hg2 = new hostgroup_class($hg->id);
		$this->ok_eq($hg2->obj['hostgroup_name'], $hg->obj['hostgroup_name'], 'The hostgroup should load correctly');
		$hg2->obj['hostgroup_name'] = 'hostgroup after rename';
		$this->ok($hg2->save_object(), 'Saving renamed hostgroup should work');
		$this->ok_empty($hg2->validation_error_list, 'The validation error list for renamed hostgroup should be empty');

		$mgmt2 = new management_pack_class($mgmt->id);
		$this->ok_eq($mgmt2->obj['management_pack_name'], $mgmt->obj['management_pack_name'], 'The management pack should load correctly');
		$this->ok_eq($mgmt2->obj['hostgroup'], $hg2->obj['hostgroup_name'], 'The hostgroup parameter for the management pack should be renamed when the group is');

		$this->ok($mgmt2->banish_object(), 'Banish object for management pack should work');
		$this->ok($hg2->banish_object(), 'Banish object for hostgroup should work');
	}

	/**
	 * if the mgmtpack points to a nonexisting hostgroup, we cannot die
	 */
	function testBrokenManagementPack() {
		global $Object_Index_Table;
		$hg = new hostgroup_class();
		$hg->obj['hostgroup_name'] = 'hostgroup name';
		$this->ok($hg->save_object(), 'Saving new hostgroup should work');
		$this->ok_empty($hg->validation_error_list, 'The validation error list for new hostgroup should be empty');

		$mgmt = new management_pack_class();
		$mgmt->obj['management_pack_name'] = 'a mgmt pack';
		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$mgmt->obj['description'] = 'a management pack';
		$mgmt->obj['icon'] = 'edit.png';
		$this->ok($mgmt->save_object(), 'Saving new mgmt pack should work');
		$this->ok_empty($mgmt->validation_error_list, 'The validation error list for new mgmt pack should be empty');

		$mgmt = new management_pack_class('a mgmt pack');
		pdo_exec_query("UPDATE management_pack SET hostgroup = 'not a hostgroup name' WHERE id = '{$mgmt->id}'");

		unset($Object_Index_Table['management_pack']);
		$tbl = get_object_index_table('management_pack');
		$this->ok(!in_array('a mgmt pack', $tbl), "The managment pack should be inaccessible when the hostgroup is broken");
		$mgmt = new management_pack_class('a mgmt pack');
		$this->ok($mgmt->id > 0, "The managment pack should be found when explicitly loading it when the hostgroup is broken");
		$this->ok(!$mgmt->save_object(), 'Saving broken mgmt pack should not work, except it does, and apparently only normalize() validates and validate_object() just goofs around for a bit, and we\'re close to release, and srsly nacoma?', TAP_TODO);

		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$this->ok($mgmt->save_object(), 'Saving fixed mgmt pack should work');
		$this->ok_empty($mgmt->validation_error_list, 'The validation error list for fixed mgmt pack should be empty');

		$this->ok($mgmt->banish_object(), 'Banish object for management pack should work');
		$this->ok($hg->banish_object(), 'Banish object for hostgroup should work');
	}

	function testRecursiveTemplate() {
		$obj = new host_class('host_defined_by_recursive_template');
		$obj->expand_object(true);
		$this->ok_eq('foo', $obj->obj['alias'], "The alias should be inherited as 'foo'");
		$obj->collapse_object(true);
		$this->ok(!isset($obj->obj['alias']), "The alias should have been compacted away");
	}

	# These pnp xml files exist: host17 (host itself, Bandwidth, Mem Usage), host14 (host itself, PING)

	function testGraphCollections() {
		$obj = new graph_collection_class('test_invalid');
		$this->ok_empty($obj->obj, 'Loading invalid collection should return empty object');
		$this->ok(!$obj->id, 'Loading invalid collection should return false id');

		$obj = new graph_collection_class();
		$obj->obj['page_name'] = 'test1';
		$obj->obj['services'] = array('host17', 'host17;Mem Usage');
		$obj->obj['use_regex'] = false;
		$this->ok($obj->save_object(), 'Saving collection should work, but got '.var_export($obj->validation_error_list, true));
		$this->ok($obj->save_object(), 'Saving same collection twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_class('test1');
		$obj->obj['services'] = array_merge($obj->obj['services'], array('host14', 'host14;PING'));
		$obj->obj['use_regex'] = false;
		$this->ok($obj->save_object(), 'Resaving collection with more services should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_class('test1');
		$obj->obj['page_name'] = 'test2';
		$this->ok($obj->save_object(), 'Saving collection with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_collection_class('test2');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_collection_class('test1');
		$this->ok_empty($obj->obj, 'Loading old name for renamed collection should return empty object');
		$this->ok(!$obj->id, 'Loading old name for renamed collection should return false id');

		$obj = new graph_collection_class('test2');
		$this->ok($obj->delete_object(), 'Object deletion should work');
		$obj = new graph_collection_class('test2');
		$this->ok_empty($obj->obj, 'Loading deleted collection should return empty object');
		$this->ok(!$obj->id, 'Loading deleted collection should return false id');
	}

	function testCombinedGraph() {
		$obj = new combined_graph_class('test_invalid');
		$this->ok_empty($obj->obj, 'Loading invalid combined graph should return empty object');
		$this->ok(!$obj->id, 'Loading invalid combined graph should return false id');

		$obj = new combined_graph_class();
		$obj->obj['title'] = 'test1';
		$obj->obj['comment'] = 'yes';
		$obj->obj['type'] = 'line2';
		$obj->obj['services'] = array('host17', 'host17;Mem Usage');
		$this->ok($obj->save_object(), 'Saving combined graph should work, but got '.var_export($obj->validation_error_list, true));
		$this->ok($obj->save_object(), 'Saving same combined graph twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_class('test1');
		$obj->obj['services'] = array_merge($obj->obj['services'], array('host14', 'host14;PING'));
		$this->ok($obj->save_object(), 'Resaving combined graph with more services should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_class('test1');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_class('test1');
		$obj->obj['title'] = 'test2';
		$this->ok($obj->save_object(), 'Saving combined graph with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new combined_graph_class('test2');
		$this->ok_eq($loaded_obj->obj, $obj->obj, sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new combined_graph_class('test1');
		$this->ok_empty($obj->obj, 'Loading old name for renamed combined graph should return empty object');
		$this->ok(!$obj->id, 'Loading old name for renamed combined graph should return false id');

		$obj = new combined_graph_class('test2');
		$this->ok($obj->delete_object(), 'Object deletion should work');
		$obj = new combined_graph_class('test2');
		$this->ok_empty($obj->obj, 'Loading deleted combined graph should return empty object');
		$this->ok(!$obj->id, 'Loading deleted combined graph should return false id');
	}

	function testGraphTemplate() {
		$obj = new graph_template_class('test_invalid');
		$this->ok_empty($obj->obj, 'Loading invalid graph template should return empty object');
		$this->ok(!$obj->id, 'Loading invalid graph template should return false id');

		$obj = new graph_template_class();
		$obj->normalize(array('check' => 'check-host-alive'));
		$obj->obj['graph_template'] = 'templates.dist/check_nt_mem.php';
		$this->ok($obj->save_object(), 'Saving graph template should work, but got '.var_export($obj->validation_error_list, true));
		$this->ok($obj->save_object(), 'Saving same graph template twice should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_class('check-host-alive');
		$this->ok_eq($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_class('check-host-alive');
		$obj->obj['graph_template'] = 'templates.dist/check_nwstat_LOAD1.php';
		$this->ok($obj->save_object(), 'Resaving graph template with different template should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_class('check-host-alive');
		$this->ok_eq($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Re-loaded object should be identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_class('check-host-alive');
		$obj->normalize(array('check' => 'check_ssh'));
		$this->ok($obj->save_object(), 'Saving graph template with new name should work, but got '.var_export($obj->validation_error_list, true));
		$loaded_obj = new graph_template_class('check_ssh');
		$this->ok_eq($loaded_obj->denormalize(), $obj->denormalize(), sprintf('Loading renamed object should become identical to what we saved, got %s vs %s', var_export($loaded_obj->obj, true), var_export($obj->obj, true)));

		$obj = new graph_template_class('check-host-alive');
		$this->ok_empty($obj->obj, 'Loading old name for renamed graph template should return empty object');
		$this->ok(!$obj->id, 'Loading old name for renamed graph template should return false id');

		$obj = new graph_template_class('check_ssh');
		$this->ok($obj->delete_object(), 'Object deletion should work, but got: '.$obj->delete_message);
		$obj = new graph_template_class('check_ssh');
		$this->ok_empty($obj->obj, 'Loading deleted graph template should return empty object');
		$this->ok(!$obj->id, 'Loading deleted graph template should return false id');
	}
}

if (empty($main_tap))
	$main_tap = new phptap('Object tests');

runUnit('ObjectTest', $main_tap);
